using System;
using System.Data;
using System.Data.SqlClient;

class SqlParametersTest
{
	private const string CONNECTION_STRING = "Server=.;" +
		" Database=Northwind; Integrated Security=true";

	private SqlConnection mDbCon;


	private void ConnectToDB()
	{
		mDbCon = new SqlConnection(CONNECTION_STRING);
		mDbCon.Open();
	}

	private void DisconnectFromDB()
	{
		if (mDbCon != null)
		{
			mDbCon.Close();
		}
	}

	private Shipper GetShipperById(decimal aId)
	{
		SqlCommand cmdSelectShipper = new SqlCommand(
			"SELECT ShipperID, CompanyName, Phone FROM Shippers " +
			"WHERE ShipperID = @id", mDbCon);
		
		SqlParameter paramId = 
			new SqlParameter("@id", SqlDbType.Decimal);
		paramId.Value = aId;
		cmdSelectShipper.Parameters.Add(paramId);

		SqlDataReader reader = cmdSelectShipper.ExecuteReader();
		using(reader)
		{
			if (reader.Read())
			{
				decimal id = (int) reader["ShipperID"];
				string name = (string) reader["CompanyName"];
				string phone = (string) reader["Phone"];
				Shipper shipper = new Shipper(id, name, phone);
				return shipper;
			}
			else
			{
				throw new ShipperException(
					String.Format("Invalid shipper (ID = {0}).", aId));
			}
		}

	}

	private decimal InsertShipper(string aName, string aPhone)
	{
		SqlCommand cmdInsertShipper = new SqlCommand(
			"INSERT INTO Shippers(CompanyName, Phone) " +
			"VALUES (@Name, @Phone)", mDbCon);
		
		SqlParameter paramName = 
			new SqlParameter("@Name", SqlDbType.NVarChar);
		paramName.Value = aName;
		cmdInsertShipper.Parameters.Add(paramName);

		SqlParameter paramPhone = 
			new SqlParameter("@Phone", SqlDbType.NVarChar);
		paramPhone.Value = aPhone;
		cmdInsertShipper.Parameters.Add(paramPhone);

		cmdInsertShipper.ExecuteNonQuery();

		SqlCommand cmdSelectIdentity = 
			new SqlCommand("SELECT @@Identity", mDbCon);

		decimal insertedRecordId = 
			(decimal) cmdSelectIdentity.ExecuteScalar();
		return insertedRecordId;
	}

	static void Main()
	{
		SqlParametersTest test = new SqlParametersTest();
		try
		{
			test.ConnectToDB();

			// Retrieve a shipper from the "Shippers" table
			Shipper shipper = test.GetShipperById(1);
			Console.WriteLine(shipper);

			// Insert new shipper in the "Shippers" table
			decimal newShipperId = 
				test.InsertShipper("", "123-456-789");
			Console.WriteLine("Inserted new shipper. " +
				"ShipperID = {0}", newShipperId);
		}
		finally
		{
		  test.DisconnectFromDB();
		}
	}

	private struct Shipper
	{
		private decimal mId;
		private string mName;
		private string mPhone;

		public Shipper(decimal aId, string aName, string aPhone)
		{
			mId = aId;
			mName = aName;
			mPhone = aPhone;
		}

		public decimal Id
		{
			get
			{
				return mId;
			}
		}

		public string Name
		{
			get
			{
				return mName;
			}
		}

		public string Phone
		{
			get
			{
				return mPhone;
			}
		}

		public override string ToString()
		{
			string text = String.Format(
				"Shipper(Id={0}, Name={1}, Phone={2})",
				mId, mName, mPhone);
			return text;
		}

	}

	private class ShipperException : ApplicationException
	{
		public ShipperException(string aName) : base(aName)
		{
		}
	}
}
