using System;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;

class DatesDemo
{
	const string CONNECTION_STRING = "Server=.; database=Northwind; Integrated Security=SSPI";
	private static SqlConnection mDbCon;

	static void Main()
	{
		DatesDemo demo = new DatesDemo();
		demo.ConnectToSqlServer();
		demo.DropMessagesTable();
		demo.CreateMessagesTable();
		demo.AddMessage("Test message 1", DateTime.Now);
		demo.AddMessage("Test message 2", DateTime.Now);
		demo.AddMessage("Test message 3", DateTime.Now);
		CultureInfo cultureBulgaria = new CultureInfo("bg-BG");
		demo.DisplayAllMessages(cultureBulgaria, "dd-MMM-yyyy HH:mm:ss");
		demo.DropMessagesTable();
		demo.DisconnectFromSqlServer();
	}

	public void ConnectToSqlServer()
	{
		mDbCon = new SqlConnection(CONNECTION_STRING);
		mDbCon.Open();
	}

	public void CreateMessagesTable()
	{
		SqlCommand cmdCreateMsgTable = new SqlCommand(
			@"CREATE TABLE Messages
			(
				MsgId int identity not null primary key,
				MsgText nvarchar(1000),
				MsgDate datetime
			)", 
			mDbCon
		);
		cmdCreateMsgTable.ExecuteNonQuery();
	}

	public void AddMessage(string aMsgText, DateTime aMsgDate)
	{
		SqlCommand cmdInsertMsg = new SqlCommand(
			"INSERT INTO Messages(MsgText, MsgDate) " +
			"VALUES (@MsgText, @MsgDate)", mDbCon);
		
		SqlParameter paramMsgText = 
			new SqlParameter("@MsgText", SqlDbType.NVarChar);
		paramMsgText.Value = aMsgText;
		cmdInsertMsg.Parameters.Add(paramMsgText);
		
		SqlParameter paramMsgDate = 
			new SqlParameter("@MsgDate", SqlDbType.DateTime);
		paramMsgDate.Value = aMsgDate;
		cmdInsertMsg.Parameters.Add(paramMsgDate);

		cmdInsertMsg.ExecuteNonQuery();

		Console.WriteLine("Inserted record in Messages table.");
	}

	public void DisplayAllMessages(CultureInfo aCultureInfo, string aFormat)
	{
		SqlCommand cmdSelectMsgs = new SqlCommand(
			"SELECT MsgText, MsgDate FROM Messages", mDbCon);
		SqlDataReader reader = cmdSelectMsgs.ExecuteReader();
		using (reader)
		{
			while (reader.Read())
			{
				string msgText = (string) reader["MsgText"];
				DateTime msgDate = (DateTime) reader["MsgDate"];
				string msgDateFormatted = msgDate.ToString(aFormat, aCultureInfo);
				Console.WriteLine("{0} - {1}", msgDateFormatted, msgText);
			}
		}
	}

	public void DropMessagesTable()
	{
		SqlCommand cmdCreateMsgTable = new SqlCommand(
			@"IF OBJECT_ID('Messages') IS NOT NULL
				DROP TABLE Messages", 
			mDbCon);
		cmdCreateMsgTable.ExecuteNonQuery();
	}

	public void DisconnectFromSqlServer()
	{
		mDbCon.Close();
	}
}
