using System;
using System.Data;
using System.Data.SqlClient;
using System.Xml;

/// <summary>
/// Demonstrates how to work with XPath expressions on DataSets
/// </summary>
class XmlDataDocumentDemo
{
	static void Main()
	{
		DataSet myDataSet = new DataSet("CustomerOrders");

		SqlConnection nWindConn = new SqlConnection(
			"Data Source=localhost;Initial Catalog=northwind;" +
			"Integrated Security=SSPI;");
		using (nWindConn)
		{
			nWindConn.Open();

			SqlDataAdapter custDA = new SqlDataAdapter(
				"SELECT * FROM Customers", nWindConn);
			custDA.Fill(myDataSet, "Customers");

			SqlDataAdapter ordersDA = new SqlDataAdapter(
				"SELECT * FROM Orders", nWindConn);
			ordersDA.Fill(myDataSet, "Orders");

			SqlDataAdapter detailsDA = new SqlDataAdapter(
				"SELECT * FROM [Order Details]", nWindConn);
			detailsDA.Fill(myDataSet, "OrderDetails");
		}

		myDataSet.Relations.Add("CustOrders",
			myDataSet.Tables["Customers"].Columns["CustomerID"],
			myDataSet.Tables["Orders"].Columns["CustomerID"]).
			Nested = true;

		myDataSet.Relations.Add("OrderDetail",
			myDataSet.Tables["Orders"].Columns["OrderID"],
			myDataSet.Tables["OrderDetails"].Columns["OrderID"],
			false).Nested = true;

		XmlDataDocument xmlDoc = new XmlDataDocument(myDataSet); 

		// Get all elements with ProductID=43
		XmlNodeList nodeList = xmlDoc.DocumentElement.SelectNodes(
			"descendant::Customers[*/OrderDetails/ProductID=43]");

		foreach (XmlNode myNode in nodeList)
		{
			DataRow customer = xmlDoc.GetRowFromElement(
				(XmlElement)myNode);
			Console.WriteLine(customer["CompanyName"]);
		}
	}
}
