using System;
using System.Collections;
using System.Data.OleDb;
using System.IO;

class ImagesInDBDemo
{
	private const string DB_CONNECTION_STRING = 
		@"Provider=Microsoft.Jet.OLEDB.4.0;" +
		@"Data Source=..\..\Images.mdb";

	private const string SOURCE_IMAGE_FILE_NAME = 
		@"..\..\logo.gif";
	private const string DEST_IMAGE_FILE_NAME = 
		@"..\..\logo-from-db.gif";

	private static byte[] ReadBinaryFile(string aFileName)
	{
		byte[] buf;
		FileStream fs = File.OpenRead(aFileName);
		using (fs)
		{
			int pos = 0;
			int length = (int) fs.Length;
			buf = new byte[length];
			while (true)
			{
				int bytesRead = fs.Read(buf, pos, length-pos);
				if (bytesRead == 0)
				{
					break;
				}
				pos += bytesRead;
			}
		}

		return buf;
	}

	private static void WriteBinaryFile(string aFileName, 
		byte[] aFileContents)
	{
		FileStream fs = File.OpenWrite(aFileName);
		using (fs)
		{
			fs.Write(aFileContents, 0, aFileContents.Length);
		}
	}

	private static string GetImageFormat(string aFileName)
	{
		FileInfo fileInfo = new FileInfo(aFileName);
		string fileExtenstion = fileInfo.Extension;
		string imageFormat = fileExtenstion.ToLower().Substring(1);
		return imageFormat;
	}

	private static int[] ListImageIdsFromDB()
	{
		OleDbConnection dbConn = new OleDbConnection(
			DB_CONNECTION_STRING);
		dbConn.Open();
		using (dbConn)
		{
			OleDbCommand cmd = new OleDbCommand(
				"SELECT ImageId FROM Images", dbConn);
			ArrayList imageIds = new ArrayList();
			OleDbDataReader reader = cmd.ExecuteReader();
			using (reader)
			{
				while (reader.Read())
				{
					int imageId = (int) reader["ImageId"];
					imageIds.Add(imageId);
				}
			}
			
			int[] imageIdArray = (int[]) imageIds.
				ToArray(typeof(int));
			return imageIdArray;
		}
	}
		
	private static void ExtractImageFromDB(
		int aImageId, out byte[] aImage, out string aImageFormat)
	{
		OleDbConnection dbConn = new OleDbConnection(
			DB_CONNECTION_STRING);
		dbConn.Open();
		using (dbConn)
		{
			OleDbCommand cmd = new OleDbCommand(
				"SELECT Image, ImageFormat FROM Images " + 
				"WHERE ImageId=@id", dbConn);
			OleDbParameter paramId = new OleDbParameter(
				"@id", OleDbType.Integer);
			paramId.Value = aImageId;
			cmd.Parameters.Add(paramId);
			OleDbDataReader reader = cmd.ExecuteReader();
			using (reader)
			{
				if (reader.Read())
				{
					aImage = (byte[]) reader["Image"];
					aImageFormat = (string) reader["ImageFormat"];
				}
				else
				{
					throw new Exception(
						String.Format("Invalid image ID={0}.", aImageId));
				}
			}
		}
	}

	private static void InsertImageToDB(byte[] aImage, 
		string aImageFormat)
	{
		OleDbConnection dbConn = new OleDbConnection(
			DB_CONNECTION_STRING);
		dbConn.Open();
		using (dbConn)
		{
			OleDbCommand cmd = new OleDbCommand(
				"INSERT INTO Images ([Image], ImageFormat) " +
				"VALUES (@image, @imageFormat)", dbConn);
			
			OleDbParameter paramImage = 
				new OleDbParameter("@image", OleDbType.Binary);
			paramImage.Value = aImage;
			cmd.Parameters.Add(paramImage);

			OleDbParameter paramImageFormat = 
				new OleDbParameter("@imageFormat", OleDbType.Char);
			paramImageFormat.Value = aImageFormat;
			cmd.Parameters.Add(paramImageFormat);

			cmd.ExecuteNonQuery();
		}
	}

	private static void DeleteAllImagesFromDB()
	{
		OleDbConnection dbConn = new OleDbConnection(
			DB_CONNECTION_STRING);
		dbConn.Open();
		using (dbConn)
		{
			OleDbCommand cmd = new OleDbCommand(
				"DELETE FROM Images", dbConn);
			cmd.ExecuteScalar();
		}
	}

	static void Main()
	{
		DeleteAllImagesFromDB();
		Console.WriteLine("Deleted all images from the DB.");

		byte[] image = ReadBinaryFile(SOURCE_IMAGE_FILE_NAME);
		string imageFormat = GetImageFormat(
			SOURCE_IMAGE_FILE_NAME);
		Console.WriteLine("Loaded image file {0}.",
			SOURCE_IMAGE_FILE_NAME);

		InsertImageToDB(image, imageFormat);
		Console.WriteLine("Inserted an image in the DB.");

		int[] imageIds = ListImageIdsFromDB();
		Console.WriteLine("There are {0} images in the DB.",
			imageIds.Length);

		int firstImageId = imageIds[0];
		byte[] imageFromDB;
		string imageFormatFromDB;
		ExtractImageFromDB(firstImageId, 
			out imageFromDB, out imageFormatFromDB);
		Console.WriteLine("Extracted first image from the DB.");

		WriteBinaryFile(DEST_IMAGE_FILE_NAME, imageFromDB);
		Console.WriteLine("Image saved to file {0}.",
			DEST_IMAGE_FILE_NAME);
	}
}
